<?php
/**
 *
 * WC_Wallee_Migration Class
 *
 * Wallee
 * This plugin will add support for all Wallee payments methods and connect the Wallee servers to your WooCommerce webshop (https://www.wallee.com).
 *
 * @category Class
 * @package  Wallee
 * @author   wallee AG (http://www.wallee.com/)
 * @license  http://www.apache.org/licenses/LICENSE-2.0 Apache Software License (ASL 2.0)
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit();
}
/**
 * Class WC_Wallee_Migration.
 *
 * @class WC_Wallee_Migration
 */
/**
 * This class handles the database setup and migration.
 */
class WC_Wallee_Migration {

	const CK_DB_VERSION = 'wc_wallee_db_version';

	/**
	 * Database migrations.
	 *
	 * @var $db_migrations database migrations.
	 */
	private static $db_migrations = array(
		'1.0.0' => 'update_1_0_0_initialize',
		'1.0.1' => 'update_1_0_1_image_url',
		'1.0.2' => 'update_1_0_2_order_allow_null',
		'1.0.3' => 'update_1_0_3_image_domain',
		'1.0.4' => 'update_1_0_4_failure_msg_and_attribute',
		'1.0.5' => 'update_1_0_5_clear_provider_transients',
		'1.0.6' => 'update_1_0_6_shorten_table_names',
	);

	/**
	 * Hook in tabs.
	 */
	public static function init() {
		add_action(
			'init',
			array(
				__CLASS__,
				'check_version',
			),
			5
		);
		add_action(
			'wpmu_new_blog',
			array(
				__CLASS__,
				'wpmu_new_blog',
			)
		);
		add_filter(
			'wpmu_drop_tables',
			array(
				__CLASS__,
				'wpmu_drop_tables',
			)
		);
		add_filter( 'plugin_row_meta', array( __CLASS__, 'plugin_row_meta' ), 10, 2 );
	}

	/**
	 * Install DB
	 *
	 * @param mixed $networkwide networkwide.
	 */
	public static function install_db( $networkwide ) {
		global $wpdb;
		if ( ! is_blog_installed() ) {
			return;
		}

		wc_maybe_define_constant( 'WC_WALLEE_INSTALLING', true );

		self::check_requirements();

		if ( function_exists( 'is_multisite' ) && is_multisite() ) {
			// check if it is a network activation - if so, run the activation function for each blog id.
			if ( $networkwide ) {
				// Get all blog ids.
				$blog_ids = $wpdb->get_col( "SELECT blog_id FROM $wpdb->blogs" );
				foreach ( $blog_ids as $blog_id ) {
					switch_to_blog( $blog_id );
					self::migrate_db();
					restore_current_blog();
				}
				return;
			}
		}
		self::migrate_db();
	}


	/**
	 * Checks if the system requirements are met
	 *
	 * Calls wp_die if requirements not met
	 */
	private static function check_requirements() {
		global $wp_version;
		require_once( ABSPATH . '/wp-admin/includes/plugin.php' );

		$errors = array();

		if ( version_compare( PHP_VERSION, WC_WALLEE_REQUIRED_PHP_VERSION, '<' ) ) {
			$errors[] = sprintf( __( "PHP %1\$s+ is required. (You're running version %2\$s)", 'woo-wallee' ), WC_WALLEE_REQUIRED_PHP_VERSION, PHP_VERSION );
		}
		if ( version_compare( $wp_version, WC_WALLEE_REQUIRED_WP_VERSION, '<' ) ) {
			$errors[] = sprintf( __( "Wordpress %1\$s+ is required. (You're running version %2\$s)", 'woo-wallee' ), WC_WALLEE_REQUIRED_WP_VERSION, $wp_version );

		}

		if ( ! is_plugin_active( 'woocommerce/woocommerce.php' ) ) {
			/* translators: %s+: version */
			$errors[] = sprintf( __( 'Woocommerce %s+ has to be active.', 'woo-wallee' ), WC_WALLEE_REQUIRED_WC_VERSION );
		} else {
			$woocommerce_data = get_plugin_data( WP_PLUGIN_DIR . '/woocommerce/woocommerce.php', false, false );

			if ( version_compare( $woocommerce_data['Version'], WC_WALLEE_REQUIRED_WC_VERSION, '<' ) ) {
				$errors[] = sprintf( __( "Woocommerce %1\$s+ is required. (You're running version %2\$s)", 'woo-wallee' ), WC_WALLEE_REQUIRED_WC_VERSION, $woocommerce_data['Version'] );
			}
			if ( version_compare( $woocommerce_data['Version'], WC_WALLEE_REQUIRED_WC_MAXIMUM_VERSION, '>' ) ) {
				$errors[] = sprintf( __( "Woocommerce version supported up to %1\$s+. (You're running version %2\$s)", 'woo-wallee' ), WC_WALLEE_REQUIRED_WC_MAXIMUM_VERSION, $woocommerce_data['Version'] );
			}
		}

		try {
			\Wallee\Sdk\Http\HttpClientFactory::getClient();
		} catch ( Exception $e ) {
			$errors[] = __( "Install the PHP cUrl extension or ensure the 'stream_socket_client' function is available." );
		}

		if ( ! empty( $errors ) ) {
			$title = __( 'Could not activate plugin wallee.', 'woo-wallee' );
			    // phpcs:ignore
			    $message = '<h1><strong>' . esc_html_e( $title ) . '</strong></h1><br/>' .
					'<h3>' . __( 'Please check the following requirements before activating:', 'woo-wallee' ) . '</h3>' .
					'<ul><li>' .
					implode( '</li><li>', $errors ) .
					'</li></ul>';
		    // phpcs:ignore
			wp_die( esc_html_e( $message ), esc_html_e( $title ), array( 'back_link' => true ) );
			return;
		}
	}

	/**
	 * Create tables if new MU blog is created
	 *
	 * @param  mixed $blog_id blog id.
	 * @param  mixed $user_id user id.
	 * @param  mixed $domain domain.
	 * @param  mixed $path path.
	 * @param  mixed $site_id site id.
	 * @param  mixed $meta meta.
	 */
	public static function wpmu_new_blog( $blog_id, $user_id, $domain, $path, $site_id, $meta ) {

		if ( is_plugin_active_for_network( 'woo-wallee/woocommerce-wallee.php' ) ) {
			switch_to_blog( $blog_id );
			self::migrate_db();
			restore_current_blog();
		}
	}

	/**
	 * Migrate the database
	 */
	private static function migrate_db() {
		$current_version = get_option( self::CK_DB_VERSION, 0 );
		foreach ( self::$db_migrations as $version => $function_name ) {
			if ( version_compare( $current_version, $version, '<' ) ) {

				call_user_func(
					array(
						__CLASS__,
						$function_name,
					)
				);

				update_option( self::CK_DB_VERSION, $version );
				$current_version = $version;
			}
		}
	}

	/**
	 * Uninstall tables when MU blog is deleted.
	 *
	 * @param  array $tables tables.
	 * @return string[]
	 */
	public static function wpmu_drop_tables( $tables ) {
		global $wpdb;

		$tables[] = $wpdb->prefix . 'wc_wallee_method_config';
		$tables[] = $wpdb->prefix . 'wc_wallee_transaction_info';
		$tables[] = $wpdb->prefix . 'wc_wallee_token_info';
		$tables[] = $wpdb->prefix . 'wc_wallee_completion_job';
		$tables[] = $wpdb->prefix . 'wc_wallee_void_job';
		$tables[] = $wpdb->prefix . 'wc_wallee_refund_job';

		return $tables;
	}

	/**
	 * Check Wallee DB version and run the migration if required.
	 *
	 * This check is done on all requests and runs if he versions do not match.
	 */
	public static function check_version() {
		try {
			$current_version = get_option( self::CK_DB_VERSION, 0 );
			$version_keys = array_keys( self::$db_migrations );
			if ( version_compare( $current_version, '0', '>' ) && version_compare( $current_version, end( $version_keys ), '<' ) ) {
				// We migrate the Db for all blogs.
				self::install_db( true );
			}
		} catch ( Exception $e ) {
			if ( is_admin() ) {
				add_action(
					'admin_notices',
					array(
						'WC_Wallee_Admin_Notices',
						'migration_failed_notices',
					)
				);
			}
		}
	}

	/**
	 * Show row meta on the plugin screen.
	 *
	 * @param   mixed $links Plugin Row Meta.
	 * @param   mixed $file  Plugin Base file.
	 * @return  array
	 */
	public static function plugin_row_meta( $links, $file ) {
		if ( WC_WALLEE_PLUGIN_BASENAME === $file ) {
			$row_meta = array(
				'docs' => '<a href="https://plugin-documentation.wallee.com/wallee-payment/woocommerce/2.1.10/docs/en/documentation.html" aria-label="' . esc_attr__( 'View Documentation', 'woo-wallee' ) . '">' . esc_html__( 'Documentation', 'woo-wallee' ) . '</a>',
			);

			return array_merge( $links, $row_meta );
		}

		return (array) $links;
	}

	/**
	 * Initialise for update 1.0.0.
	 *
	 * @throws Exception Exception.
	 */
	public static function update_1_0_0_initialize() {
		global $wpdb;

		$result = $wpdb->query(
			"CREATE TABLE IF NOT EXISTS {$wpdb->prefix}wc_wallee_method_config(
				`id` int(10) unsigned NOT NULL AUTO_INCREMENT,
				`state` varchar(255) COLLATE utf8_unicode_ci NOT NULL,
				`space_id` bigint(20) unsigned NOT NULL,
				`created_at` datetime NOT NULL,
				`updated_at` datetime NOT NULL,
				`configuration_id` bigint(20) unsigned NOT NULL,
				`configuration_name` varchar(150) COLLATE utf8_unicode_ci NOT NULL,
				`title` longtext COLLATE utf8_unicode_ci,
				`description` longtext COLLATE utf8_unicode_ci,
				`image` varchar(512) COLLATE utf8_unicode_ci DEFAULT NULL,
				PRIMARY KEY (`id`),
				UNIQUE KEY `unq_space_id_configuration_id` (`space_id`,`configuration_id`),
				KEY `idx_space_id` (`space_id`),
				KEY `idx_configuration_id` (`configuration_id`)
				) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;"
		);

		if ( false === $result ) {
			throw new Exception( $wpdb->last_error );
		}

		$result = $wpdb->query(
			"CREATE TABLE IF NOT EXISTS {$wpdb->prefix}wc_wallee_transaction_info(
				`id` int(10) unsigned NOT NULL AUTO_INCREMENT,
				`transaction_id` bigint(20) unsigned NOT NULL,
				`state` varchar(255) COLLATE utf8_unicode_ci NOT NULL,
				`space_id` bigint(20) unsigned NOT NULL,
				`space_view_id` bigint(20) unsigned DEFAULT NULL,
				`language` varchar(255) COLLATE utf8_unicode_ci NOT NULL,
				`currency` varchar(255) COLLATE utf8_unicode_ci NOT NULL,
				`created_at` datetime NOT NULL,
				`updated_at` datetime NOT NULL,
				`authorization_amount` decimal(19,8) NOT NULL,
				`image` varchar(512) COLLATE utf8_unicode_ci DEFAULT NULL,
				`labels` longtext COLLATE utf8_unicode_ci,
				`payment_method_id` bigint(20) unsigned DEFAULT NULL,
				`connector_id` bigint(20) unsigned DEFAULT NULL,
				`order_id` int(10) unsigned NOT NULL,
				`failure_reason` longtext COLLATE utf8_unicode_ci,
				`locked_at` datetime,
				PRIMARY KEY (`id`),
				UNIQUE KEY `unq_transaction_id_space_id` (`transaction_id`,`space_id`),
				UNIQUE KEY `unq_order_id` (`order_id`)
				) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci"
		);

		if ( false === $result ) {
			throw new Exception( $wpdb->last_error );
		}

		$result = $wpdb->query(
			"CREATE TABLE IF NOT EXISTS {$wpdb->prefix}wc_wallee_token_info(
				`id` int(10) unsigned NOT NULL AUTO_INCREMENT,
				`token_id` bigint(20) unsigned NOT NULL,
				`state` varchar(255) COLLATE utf8_unicode_ci NOT NULL,
				`space_id` bigint(20) unsigned NOT NULL,
				`name` varchar(255) COLLATE utf8_unicode_ci NOT NULL,
				`created_at` datetime NOT NULL,
				`updated_at` datetime NOT NULL,
				`customer_id` int(10) unsigned NOT NULL,
				`payment_method_id` int(10) unsigned NOT NULL,
				`connector_id` bigint(20) unsigned DEFAULT NULL,
				PRIMARY KEY (`id`),
				UNIQUE KEY `unq_transaction_id_space_id` (`token_id`,`space_id`),
				KEY `idx_customer_id` (`customer_id`),
				KEY `idx_payment_method_id` (`payment_method_id`),
				KEY `idx_connector_id` (`connector_id`)
				) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;"
		);

		if ( false === $result ) {
			throw new Exception( $wpdb->last_error );
		}
		$result = $wpdb->query(
			"CREATE TABLE IF NOT EXISTS {$wpdb->prefix}wc_wallee_completion_job(
				`id` int(10) unsigned NOT NULL AUTO_INCREMENT,
				`state` varchar(255) COLLATE utf8_unicode_ci NOT NULL,
				`completion_id` bigint(20) unsigned,
				`transaction_id` bigint(20) unsigned NOT NULL,
				`space_id` bigint(20) unsigned NOT NULL,
				`order_id` bigint(20) unsigned NOT NULL,
				`amount` decimal(19,8) NOT NULL,
				`created_at` datetime NOT NULL,
				`updated_at` datetime NOT NULL,
				`restock` varchar(1) COLLATE utf8_unicode_ci,
				`items` longtext COLLATE utf8_unicode_ci,
				`failure_reason` longtext COLLATE utf8_unicode_ci,			
				PRIMARY KEY (`id`),
				KEY `idx_transaction_id_space_id` (`transaction_id`,`space_id`),
				KEY `idx_completion_id_space_id` (`completion_id`,`space_id`)
				) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci"
		);

		if ( false === $result ) {
			throw new Exception( $wpdb->last_error );
		}

		$result = $wpdb->query(
			"CREATE TABLE IF NOT EXISTS {$wpdb->prefix}wc_wallee_void_job(
				`id` int(10) unsigned NOT NULL AUTO_INCREMENT,
				`state` varchar(255) COLLATE utf8_unicode_ci NOT NULL,
				`void_id` bigint(20) unsigned,
				`transaction_id` bigint(20) unsigned NOT NULL,
				`space_id` bigint(20) unsigned NOT NULL,
				`order_id` bigint(20) unsigned NOT NULL,
				`created_at` datetime NOT NULL,
				`updated_at` datetime NOT NULL,
				`restock` varchar(1) COLLATE utf8_unicode_ci,
				`failure_reason` longtext COLLATE utf8_unicode_ci,
				PRIMARY KEY (`id`),
				KEY `idx_transaction_id_space_id` (`transaction_id`,`space_id`),
				KEY `idx_void_id_space_id` (`void_id`,`space_id`)
				) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci"
		);

		if ( false === $result ) {
			throw new Exception( $wpdb->last_error );
		}

		$result = $wpdb->query(
			"CREATE TABLE IF NOT EXISTS {$wpdb->prefix}wc_wallee_refund_job(
				`id` int(10) unsigned NOT NULL AUTO_INCREMENT,
				`state` varchar(255) COLLATE utf8_unicode_ci NOT NULL,
				`wc_refund_id` bigint(20) unsigned NOT NULL,
				`external_id` varchar(100) NOT NULL,
				`transaction_id` bigint(20) unsigned NOT NULL,
				`space_id` bigint(20) unsigned NOT NULL,
				`order_id` bigint(20) unsigned NOT NULL,
				`created_at` datetime NOT NULL,
				`updated_at` datetime NOT NULL,
				`refund` longtext COLLATE utf8_unicode_ci,
				`failure_reason` longtext COLLATE utf8_unicode_ci,
				PRIMARY KEY (`id`),
				KEY `idx_transaction_id_space_id` (`transaction_id`,`space_id`),
				KEY `idx_external_id_space_id` (`external_id`,`space_id`)
				) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci"
		);

		if ( false === $result ) {
			throw new Exception( $wpdb->last_error );
		}

	}

	/**
	 * Update image url for update 1.0.1.
	 *
	 * @throws Exception Exception.
	 */
	public static function update_1_0_1_image_url() {
		global $wpdb;
		$result = $wpdb->query(
			"ALTER TABLE `{$wpdb->prefix}wc_wallee_method_config` CHANGE `image` `image` VARCHAR(2047) CHARACTER SET utf8 COLLATE utf8_unicode_ci NULL DEFAULT NULL;"
		);
		if ( false === $result ) {
			throw new Exception( $wpdb->last_error );
		}

		$result = $wpdb->query(
			"ALTER TABLE `{$wpdb->prefix}wc_wallee_transaction_info` CHANGE `image` `image` VARCHAR(2047) CHARACTER SET utf8 COLLATE utf8_unicode_ci NULL DEFAULT NULL;"
		);
		if ( false === $result ) {
			throw new Exception( $wpdb->last_error );
		}
	}

	/**
	 * Allow order NULL for update 1.0.2.
	 *
	 * @throws Exception Exception.
	 */
	public static function update_1_0_2_order_allow_null() {
		global $wpdb;
		$result = $wpdb->query(
			"ALTER TABLE `{$wpdb->prefix}wc_wallee_transaction_info` CHANGE `order_id` `order_id` int(10) unsigned NULL DEFAULT NULL;"
		);
		if ( false === $result ) {
			throw new Exception( $wpdb->last_error );
		}

		$result = $wpdb->query( "SHOW COLUMNS FROM `{$wpdb->prefix}wc_wallee_transaction_info` LIKE 'order_mapping_id'" );
		if ( 0 == $result ) {
			$result = $wpdb->query(
				"ALTER TABLE `{$wpdb->prefix}wc_wallee_transaction_info` ADD `order_mapping_id` int(10) unsigned NULL AFTER order_id;"
			);
			if ( false === $result ) {
				throw new Exception( $wpdb->last_error );
			}
		}
		$result = $wpdb->query(
			"ALTER TABLE `{$wpdb->prefix}wc_wallee_token_info` CHANGE `customer_id` `customer_id` int(10) unsigned NULL DEFAULT NULL;"
		);

		if ( false === $result ) {
			throw new Exception( $wpdb->last_error );
		}
	}

	/**
	 * Add image base fpr update 1.0.3.
	 *
	 * @throws Exception Exception.
	 */
	public static function update_1_0_3_image_domain() {
		global $wpdb;

		$result = $wpdb->query( "SHOW COLUMNS FROM `{$wpdb->prefix}wc_wallee_method_config` LIKE 'image_base'" );
		if ( 0 == $result ) {
			$result = $wpdb->query(
				"ALTER TABLE `{$wpdb->prefix}wc_wallee_method_config` ADD COLUMN `image_base` VARCHAR(2047) CHARACTER SET utf8 COLLATE utf8_unicode_ci NULL DEFAULT NULL AFTER image;"
			);
			if ( false === $result ) {
				throw new Exception( $wpdb->last_error );
			}
		}

		$result = $wpdb->query( "SHOW COLUMNS FROM `{$wpdb->prefix}wc_wallee_transaction_info` LIKE 'image_base'" );
		if ( 0 == $result ) {
			$result = $wpdb->query(
				"ALTER TABLE `{$wpdb->prefix}wc_wallee_transaction_info` ADD COLUMN `image_base` VARCHAR(2047) CHARACTER SET utf8 COLLATE utf8_unicode_ci NULL DEFAULT NULL AFTER image;"
			);
			if ( false === $result ) {
				throw new Exception( $wpdb->last_error );
			}
		}
	}

	/**
	 * Add failure message and attribute for update 1.0.4.
	 *
	 * @throws Exception Exception.
	 */
	public static function update_1_0_4_failure_msg_and_attribute() {
		global $wpdb;

		$result = $wpdb->query( "SHOW COLUMNS FROM `{$wpdb->prefix}wc_wallee_transaction_info` LIKE 'user_failure_message'" );
		if ( 0 == $result ) {
			$result = $wpdb->query(
				"ALTER TABLE `{$wpdb->prefix}wc_wallee_transaction_info` ADD COLUMN `user_failure_message` VARCHAR(2047) CHARACTER SET utf8 COLLATE utf8_unicode_ci NULL DEFAULT NULL AFTER failure_reason;"
			);
			if ( false === $result ) {
				throw new Exception( $wpdb->last_error );
			}
		}
		// Do not use foreign keys to reference attribute to cascade deletion, as some shop still run with MyISAM enginge as default.
		$result = $wpdb->query(
			"CREATE TABLE IF NOT EXISTS {$wpdb->prefix}wc_wallee_attribute_options(
				`id` int(10) unsigned NOT NULL AUTO_INCREMENT,
                `attribute_id` bigint(20) UNSIGNED NOT NULL,
				`send` varchar(1) COLLATE utf8_unicode_ci,
				PRIMARY KEY (`id`),
				UNIQUE `unq_attribute_id` (`attribute_id`)
				) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci"
		);
		if ( false === $result ) {
			throw new Exception( $wpdb->last_error );
		}
	}

	/**
	 * Delete provider transients.
	 */
	public static function update_1_0_5_clear_provider_transients() {
		WC_Wallee_Helper::instance()->delete_provider_transients();
	}

	/**
	 * Shorten table names on update 1.0.6.
	 *
	 * @throws Exception Exception.
	 */
	public static function update_1_0_6_shorten_table_names() {
		global $wpdb;
		// old table names format => new table names format.
		$table_rename_map = array(
			"{$wpdb->prefix}woocommerce_wallee_attribute_options" => "{$wpdb->prefix}wc_wallee_attribute_options",
			"{$wpdb->prefix}woocommerce_wallee_completion_job" => "{$wpdb->prefix}wc_wallee_completion_job",
			"{$wpdb->prefix}woocommerce_wallee_method_configuration" => "{$wpdb->prefix}wc_wallee_method_config",
			"{$wpdb->prefix}woocommerce_wallee_refund_job" => "{$wpdb->prefix}wc_wallee_refund_job",
			"{$wpdb->prefix}woocommerce_wallee_token_info" => "{$wpdb->prefix}wc_wallee_token_info",
			"{$wpdb->prefix}woocommerce_wallee_transaction_info" => "{$wpdb->prefix}wc_wallee_transaction_info",
			"{$wpdb->prefix}woocommerce_wallee_void_job" => "{$wpdb->prefix}wc_wallee_void_job",
		);

		// rollback table rename if there are issues.
		$wpdb->query( 'START TRANSACTION' );

		foreach ( $table_rename_map as $key => $value ) {
			// check old table exists.
			// phpcs:ignore
			$old_table_result = $wpdb->get_row( "SHOW TABLES WHERE Tables_in_{$wpdb->dbname} = '{$key}'" );

			if ( $old_table_result ) {
				// check new table doesn't exist, if it does there's a problem!
				// phpcs:ignore
				$new_table_result = $wpdb->get_row( "SHOW TABLES WHERE Tables_in_{$wpdb->dbname} = '{$value}'" );

				if ( ! $new_table_result ) {
					// phpcs:ignore
					$result = $wpdb->query( "RENAME TABLE {$key} TO {$value}" );
					if ( false === $result ) {
						throw new Exception( $wpdb->last_error );
					}
				}
			}
		}

		$wpdb->query( 'COMMIT' );
	}
}

WC_Wallee_Migration::init();
